#!/usr/bin/perl -w

# goggify.pl -- Gtk2 GUI Interface for Oggify
# Copyright (c) 2005,2006 Scott Paul Robertson (spr@mahonri5.net)
#
# This is part of Oggify (http://spr.mahonri5.net/oggify/)
#
# Oggify is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version
# 
# Oggify is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with Oggify; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

# TODO: any variables found in the worker thread need to not be globals, and
# instead be set on a local level and exported back to the module. or something.
# Basically, all settings remain at the default, well, most of them do at least.

use strict;

use Config;
BEGIN {
    $Config{useithreads} or die "Recompile perl with ithreads to use this program.";
}

use threads;
use threads::shared;
use Thread::Queue;
use FindBin;
use lib "$FindBin::RealBin";

use Oggify qw($Flac_Tree_Head $Output_Tree_Head $Mode $Not_Mode $Quality
            $Verbose $Nice_Value $Enc_Type $Symlink $Force
            $Tmp_File %Encode @Purge %Reencode @Lim_Purge);

share($Mode);
share($Not_Mode);
share($Quality);
share($Verbose);
share($Nice_Value);
share($Enc_Type);
share($Tmp_File);
share($Force);

# Start the thread!
our $hold : shared;
our $queue : shared = Thread::Queue->new;
our $locked : shared = 0;
my $thr = threads->new (\&command_runner);

use File::Find;
use Gtk2 '-init';
use Gtk2::SimpleList;
use constant TRUE => 1;
use constant FALSE => 0;

my $version_num = $Oggify::VERSION;

# Global variables {{{1
my $ready = FALSE;
my $purge_ok = 0;
#}}}1

load_rc_file();

# Gui Initialization code {{{1
my $window = Gtk2::Window->new;
$window->set_title ('Gui-Oggify');
$window->signal_connect (delete_event => sub { quit() });
$window->set_default_size (900, 300);

# Vbox
my $vbox = Gtk2::VBox->new (FALSE, 0);
$window->add ($vbox);

# Status Bar
my $statusbar = Gtk2::Statusbar->new;
$statusbar->push (0, 'Initalized');

# Menu {{{2
my $menubar = Gtk2::MenuBar->new;

# File Menu
my $file_menu = Gtk2::Menu->new;
# Preferences Option
my $file_menu_preferences = Gtk2::ImageMenuItem->new_from_stock ('gtk-preferences', undef);
$file_menu_preferences->signal_connect (activate => sub {make_preferences()});
$file_menu->append ($file_menu_preferences);
# Seperator
$file_menu->append (Gtk2::SeparatorMenuItem->new());
# Quit Option
my $file_menu_exit = Gtk2::ImageMenuItem->new_from_stock ('gtk-quit', undef);
$file_menu_exit->signal_connect (activate => sub { quit() });
$file_menu->append ($file_menu_exit);
# Box it up and add it to the menu bar
my $file_menu_item = Gtk2::MenuItem->new ('_File');
$file_menu_item->set_submenu ($file_menu);
$menubar->append ($file_menu_item);

# Help Menu
my $help_menu = Gtk2::Menu->new;
# Help option
my $help_menu_help = Gtk2::ImageMenuItem->new_from_stock ('gtk-help', undef);
$help_menu_help->signal_connect (activate => sub { help_dialog() });
$help_menu->append ($help_menu_help);
# About option
my $help_menu_about = Gtk2::ImageMenuItem->new_from_stock ('gtk-about', undef);
$help_menu_about->signal_connect (activate => sub { about_dialog() });
$help_menu->append ($help_menu_about);
# Box it up and add it to the menubar
my $help_menu_item = Gtk2::MenuItem->new ('_Help');
$help_menu_item->set_submenu ($help_menu);
$menubar->append ($help_menu_item);
#}}}2

# Tool Bar {{{2
my $tt = Gtk2::Tooltips->new;
my $toolbar = Gtk2::Toolbar->new;
$toolbar->set_icon_size ('small-toolbar');
$toolbar->set_show_arrow (FALSE);

# Setup the Run button
my $toolbar_run = Gtk2::ToolButton->new (undef, 'Run');
$toolbar_run->set_tooltip ($tt, 'Perform The Detailed Actions', '');
$toolbar_run->signal_connect  (clicked => sub { run() });
$toolbar_run->set_sensitive ($ready);
$toolbar->insert ($toolbar_run, -1);

$toolbar->insert (Gtk2::SeparatorToolItem->new ,-1 );

# Purge Enabling Button
my $toolbar_purge = Gtk2::ToggleToolButton->new;
$toolbar_purge->set_label ('Purge');
$toolbar_purge->set_tooltip ($tt, 'Enable Purge', '');
$toolbar_purge->set_active (FALSE);
$toolbar_purge->signal_connect (toggled => sub {
    ($toolbar_purge->get_active) and add_purge() and 
        ($statusbar->push (1, 'Purge On'));
    ($toolbar_purge->get_active) or remove_purge() or
        ($statusbar->push (1, 'Purge Off'));
});
$toolbar->insert ($toolbar_purge, -1);

# Encode Enabling Button
my $toolbar_encode = Gtk2::ToggleToolButton->new;
$toolbar_encode->set_label ('Encode');
$toolbar_encode->set_tooltip ($tt, 'Enable Encoding', '');
$toolbar_encode->set_active (TRUE);
$toolbar_encode->signal_connect  (toggled => sub {
    ($toolbar_encode->get_active) and add_encode() and
        ($statusbar->push (1, 'Encode On'));
    ($toolbar_encode->get_active) or remove_encode() or
        ($statusbar->push (1, 'Encode Off'));
});
$toolbar->insert ($toolbar_encode, -1);

# Re-Tag Enabling Button
my $toolbar_retag = Gtk2::ToggleToolButton->new;
$toolbar_retag->set_label ('Re-Tag');
$toolbar_retag->set_tooltip ($tt, 'Enable Re-Tagging', '');
$toolbar_retag->set_active (FALSE);
$toolbar_retag->signal_connect (toggled => sub {
    ($toolbar_retag->get_active) and add_re('tag') and
        ($statusbar->push (1, 'Re-Tag On'));
    ($toolbar_retag->get_active) or remove_re('tag') or
        ($statusbar->push (1, 'Re-Tag Off'));
});
$toolbar->insert ($toolbar_retag, -1);

# Re-Encode Enabling Button
my $toolbar_reencode = Gtk2::ToggleToolButton->new;
$toolbar_reencode->set_label ('Re-Encode');
$toolbar_reencode->set_tooltip ($tt, 'Enable Re-Encoding', '');
$toolbar_reencode->set_active (FALSE);
$toolbar_reencode->signal_connect (toggled => sub {
    ($toolbar_reencode->get_active) and add_re('encode') and
        ($statusbar->push (1, 'Re-Encoding On'));
    ($toolbar_reencode->get_active) or remove_re('encode') or
        ($statusbar->push (1, 'Re-Encoding Off'));
});
$toolbar->insert ($toolbar_reencode, -1);

# Clean enabling button (purge miss-encoded files)
my $toolbar_lpurge = Gtk2::ToggleToolButton->new;
$toolbar_lpurge->set_label ('Clean');
$toolbar_lpurge->set_tooltip ($tt, 'Clean out miss-encoded files', '');
$toolbar_lpurge->set_active (TRUE);
$toolbar_lpurge->signal_connect (toggled => sub {
        ($toolbar_lpurge->get_active) and add_lim_purge() and
            ($statusbar->push (1, 'Clean On'));
        ($toolbar_lpurge->get_active) or remove_lim_purge() or
            ($statusbar->push (1, 'Clean Off'));
    });
$toolbar->insert ($toolbar_lpurge, -1);

$toolbar->insert (Gtk2::SeparatorToolItem->new ,-1 );

# Re-Process Button
my $toolbar_reprocess = Gtk2::ToolButton->new (undef, 'Scan');
$toolbar_reprocess->set_tooltip ($tt, 'Re-Scan the Directories', '');
$toolbar_reprocess->signal_connect (clicked => sub {
        %Encode = ();
        @Purge = ();
        %Reencode = ();
        @Lim_Purge = ();
        Oggify::process();
        setup_list_display();
    });
$toolbar->insert ($toolbar_reprocess, -1);

#}}}2

# List {{{2
my $guilist = Gtk2::SimpleList->new (
    'Source'    => 'text',
    'Action'    => 'text',
    'Output'    => 'text',
);

map { $_->set_resizable (TRUE) } $guilist->get_columns;
map { $_->set_sizing ('fixed') } $guilist->get_columns;
my @columns = $guilist->get_columns;
$columns[0]->set_fixed_width (410);
$columns[1]->set_fixed_width (60);
$columns[2]->set_fixed_width (410);

$columns[0]->signal_connect ( clicked => sub {$statusbar->push (0, "Source")});

# how to adjust text alignment..
#@stuff = $columns[0]->get_cell_renderers;
#$rendr_0 = shift @stuff;
#$rendr_0->set (xalign => 0.95);
$columns[1]->set_alignment (0.5);
my @stuff = $columns[1]->get_cell_renderers;
my $rendr_1 = shift @stuff;
$rendr_1->set (xalign => 0.5);

$guilist->set_headers_clickable (TRUE);
$guilist->set_rules_hint (TRUE);
# }}}2

# Scrolling window
my $scrolled = Gtk2::ScrolledWindow->new;
$scrolled->set_policy ('automatic', 'automatic');
$scrolled->add ($guilist);

# Fill VBox
$vbox->pack_start ($menubar,FALSE,FALSE,0);
$vbox->pack_start ($toolbar,FALSE,FALSE,0);
$vbox->pack_start ($scrolled,TRUE,TRUE,0);
$vbox->pack_start ($statusbar,FALSE,FALSE,0);

$window->show_all;

#}}}1

setpgrp(0,0);
$SIG{QUIT} = \&Oggify::sigdie_handler;
$SIG{INT} = \&Oggify::sigdie_handler;

if (verify_arguments()) {
    print "starting at $Flac_Tree_Head\nGoing to: $Output_Tree_Head\n";
    Oggify::process();
    add_encode();
    add_lim_purge();
} else {
    make_preferences();
}

Gtk2->main;

exit(0);


# List editting functions {{{1
sub add_purge {
    for (@Purge) {
        $purge_ok = 1;
        if (-d $_) {
            my %options = ('wanted' => \&purge_wanted, 'bydepth' => 1,
                'follow' => 1);
            find(\%options, ($_));
        }
        push @{$guilist->{data}}, ['-', 'purge', $_] if ($purge_ok || $Force);
    }

    return TRUE;
}

sub purge_wanted {
    if ( -e $_ && ! -d $_ ) {
        $purge_ok = 0;
    }
}

sub remove_purge {
    for (my $i = 0; $i < @{$guilist->{data}};) {
        if ($guilist->{data}[$i][1] eq 'purge') {
            splice @{ $guilist->{data} }, $i, 1;
        } else {
            $i++;
        }
    }
    return FALSE;
}

sub add_lim_purge {
    for (@Lim_Purge) {
        push @{$guilist->{data}}, ['-', 'purge', $_];
    }
    return TRUE;
}

sub remove_lim_purge {
    remove_purge();
    setup_list_display();
    return FALSE;
}

sub add_re {
    my ($Mode) = @_;

    my @key_list = keys %Reencode;
    @key_list = sort @key_list;

    if ($Mode eq 'tag') {
        for (@key_list) {
            push @{$guilist->{data}}, [$_, 're-tag', $Reencode{$_}];
        }
    } else {
        for (@key_list) {
            push @{$guilist->{data}}, [$_, 're-encode', $Reencode{$_}];
        }
    }

    return TRUE;
}

sub remove_re {
    my ($Mode) = @_;
    $Mode = 're-' . $Mode;

    for (my $i = 0; $i < @{$guilist->{data}};) {
        if ($guilist->{data}[$i][1] eq $Mode) {
            splice @{$guilist->{data}}, $i, 1;
        } else {
            $i++;
        }
    }
    return FALSE;
}

sub add_encode {
    my @key_list = keys %Encode;
    @key_list = sort @key_list;
    for (@key_list) {
        push @{$guilist->{data}}, [$_, 'encode', $Encode{$_}];
    }
    return TRUE;
}

sub remove_encode {
    for (my $i = 0; $i < @{$guilist->{data}};) {
        if ($guilist->{data}[$i][1] eq 'encode') {
            splice @{$guilist->{data}}, $i, 1;
        } else {
            $i++;
        }
    }
    return FALSE;
}
#}}}1

# Gui Helper Code {{{1
# Make Preferences Dialog {{{2
sub make_preferences {
    my $pref_dialog = Gtk2::Dialog->new ('Preferences', $window,
        'destroy-with-parent', 'gtk-cancel' => 'cancel',
        'gtk-save' => 'accept');

    my $pref_hbox = Gtk2::HBox->new (FALSE,10);
    my $pref_left_vbox = Gtk2::VBox->new (TRUE,5);
    my $pref_right_vbox = Gtk2::VBox->new (TRUE,5);

    $pref_hbox->pack_start ($pref_left_vbox,TRUE,FALSE,0);
    $pref_hbox->pack_start (Gtk2::VSeparator->new,FALSE,FALSE,0);
    $pref_hbox->pack_start ($pref_right_vbox,TRUE,FALSE,0);
    $pref_dialog->vbox->pack_start ($pref_hbox,FALSE,FALSE,0);

    # Entry for Flac Tree Head {{{3
    my $fth_entry_table = Gtk2::Table->new (1,3,FALSE);
    $fth_entry_table->set_col_spacings (5);
    my $fth_entry_label = Gtk2::Label->new_with_mnemonic ("_Flac Tree:");
    $fth_entry_label->set_alignment (1,0.5);
    my $fth_entry = Gtk2::Entry->new;
    $fth_entry_label->set_mnemonic_widget ($fth_entry);
    if ($Flac_Tree_Head) {
        $fth_entry->set_text ($Flac_Tree_Head);
    }
    my $fth_open_button = Gtk2::Button->new ();
    $fth_open_button->add (Gtk2::Image->new_from_icon_name ('gtk-open',
            'small-toolbar'));
    $fth_open_button->signal_connect (clicked => sub {
            my $hold = choose_folder ($fth_entry->get_text); 
            if ($hold) {
                $fth_entry->set_text ($hold);
            }
        });

    $fth_entry_table->attach_defaults ($fth_entry_label,0,1,0,1);
    $fth_entry_table->attach_defaults ($fth_entry,1,2,0,1);
    $fth_entry_table->attach_defaults ($fth_open_button,2,3,0,1);
    #}}}3

    # Entry for Output Tree Head {{{3
    my $oth_entry_table = Gtk2::Table->new (1,3,FALSE);
    $oth_entry_table->set_col_spacings (5);
    my $oth_entry_label = Gtk2::Label->new_with_mnemonic ("_Output Tree:");
    $oth_entry_label->set_alignment (1,0.5);
    my $oth_entry = Gtk2::Entry->new;
    $oth_entry_label->set_mnemonic_widget ($oth_entry);
    if ($Output_Tree_Head) {
        $oth_entry->set_text ($Output_Tree_Head);
    }
    my $oth_open_button = Gtk2::Button->new ();
    $oth_open_button->add (Gtk2::Image->new_from_icon_name ('gtk-open',
            'small-toolbar'));
    $oth_open_button->signal_connect (clicked => sub {
            my $hold = choose_folder ($oth_entry->get_text); 
            if ($hold) {
                $oth_entry->set_text ($hold);
            }
        });

    $oth_entry_table->attach_defaults ($oth_entry_label,0,1,0,1);
    $oth_entry_table->attach_defaults ($oth_entry,1,2,0,1);
    $oth_entry_table->attach_defaults ($oth_open_button,2,3,0,1);
    #}}}3

    # Radio Buttons for Encoding Type (mp3) {{{3
    my $vbr_enc_radio = Gtk2::RadioButton->new (undef,'vbr');
    my @group = $vbr_enc_radio->get_group;
    my $cbr_enc_radio = Gtk2::RadioButton->new (@group,'cbr');
    @group = $vbr_enc_radio->get_group;
    my $abr_enc_radio = Gtk2::RadioButton->new (@group,'abr');
    @group = $vbr_enc_radio->get_group;
    my $enc_radio_table = Gtk2::Table->new (1,4,FALSE);
    $enc_radio_table->set_col_spacings (5);
    my $enc_radio_label = Gtk2::Label->new_with_mnemonic ("mp3 Encoding _Type:");
    $enc_radio_label->set_mnemonic_widget ($vbr_enc_radio);
    my @enc_group = @group;

    # Connect Signals
    $vbr_enc_radio->signal_connect (clicked => sub {
            ($vbr_enc_radio->get_active) and ($Enc_Type = "vbr"); });
    $cbr_enc_radio->signal_connect (clicked => sub {
            ($cbr_enc_radio->get_active) and ($Enc_Type = "cbr"); });
    $abr_enc_radio->signal_connect (clicked => sub {
            ($abr_enc_radio->get_active) and ($Enc_Type = "abr"); });

    if ($Enc_Type eq "vbr") { $vbr_enc_radio->set_active (TRUE); }
    elsif ($Enc_Type eq "cbr") { $cbr_enc_radio->set_active (TRUE); }
    elsif ($Enc_Type eq "abr") { $abr_enc_radio->set_active (TRUE); }
    if ($Mode eq ".ogg") {
        $vbr_enc_radio->set_sensitive (FALSE);
        $abr_enc_radio->set_sensitive (FALSE);
        $cbr_enc_radio->set_sensitive (FALSE);
    }

    $enc_radio_table->attach_defaults ($enc_radio_label,0,1,0,1);
    $enc_radio_table->attach_defaults ($vbr_enc_radio,1,2,0,1);
    $enc_radio_table->attach_defaults ($cbr_enc_radio,2,3,0,1);
    $enc_radio_table->attach_defaults ($abr_enc_radio,3,4,0,1);
    #}}}3

    # Radio Buttons for Encoding Mode {{{3
    my $ogg_mode_radio = Gtk2::RadioButton->new (undef,'ogg');
    @group = $ogg_mode_radio->get_group;
    my $mp3_mode_radio = Gtk2::RadioButton->new (@group,'mp3');
    @group = $ogg_mode_radio->get_group;
    my $Mode_radio_table = Gtk2::Table->new (1,3,FALSE);
    $Mode_radio_table->set_col_spacings (5);
    my $Mode_radio_table_label = Gtk2::Label->new_with_mnemonic ("Encoding _Mode");
    $Mode_radio_table_label->set_alignment (1,0.5);
    $Mode_radio_table_label->set_mnemonic_widget ($ogg_mode_radio);
    
    # Connect Signals.
    $ogg_mode_radio->signal_connect (clicked => sub {
            if ($ogg_mode_radio->get_active) {
                $Mode = ".ogg";
                $Not_Mode = ".mp3";
                $vbr_enc_radio->set_sensitive (FALSE);
                $cbr_enc_radio->set_sensitive (FALSE);
                $abr_enc_radio->set_sensitive (FALSE);
            }
        });
    $mp3_mode_radio->signal_connect (clicked => sub {
            if ($mp3_mode_radio->get_active) {
                $Mode = ".mp3";
                $Not_Mode = ".ogg";
                $vbr_enc_radio->set_sensitive (TRUE);
                $cbr_enc_radio->set_sensitive (TRUE);
                $abr_enc_radio->set_sensitive (TRUE);
            }
        });

    # Assign Pre-defined Values
    if ($Mode eq ".mp3") {
        $mp3_mode_radio->set_active (TRUE);
    } else {
        $ogg_mode_radio->set_active (TRUE);
    }

    $Mode_radio_table->attach_defaults ($Mode_radio_table_label,0,1,0,1);
    $Mode_radio_table->attach_defaults ($ogg_mode_radio,1,2,0,1);
    $Mode_radio_table->attach_defaults ($mp3_mode_radio,2,3,0,1);
    #}}}3

    # Spinner for Nice value {{{3
    my $nice_adj = Gtk2::Adjustment->new ($Nice_Value,0,20,1,1,0);
    my $nice_spinner = Gtk2::SpinButton->new ($nice_adj,0.0,0);
    $nice_spinner->set_alignment (1.0);
    $nice_spinner->set_editable (FALSE);
    my $nice_table = Gtk2::Table->new (1,2,FALSE);
    $nice_table->set_col_spacings (5);
    my $nice_label = Gtk2::Label->new_with_mnemonic ("_Nice Value:");
    $nice_label->set_alignment (1.0,0.5);
    $nice_label->set_mnemonic_widget ($nice_spinner);

    $nice_table->attach_defaults ($nice_label,0,1,0,1);
    $nice_table->attach_defaults ($nice_spinner,1,2,0,1);
    #}}}3

    # Spinner for Quality {{{3
    my $Quality_adj = Gtk2::Adjustment->new ($Quality,0,10,1,1,0);
    my $Quality_spinner = Gtk2::SpinButton->new ($Quality_adj,0.0,0);
    $Quality_spinner->set_alignment (1.0);
    $Quality_spinner->set_editable (FALSE);
    my $Quality_table = Gtk2::Table->new (1,2,FALSE);
    $Quality_table->set_col_spacings (5);
    my $Quality_label = Gtk2::Label->new_with_mnemonic ("_Quality:");
    $Quality_label->set_mnemonic_widget ($Quality_spinner);
    $Quality_label->set_alignment (1.0,0.5);

    $Quality_table->attach_defaults ($Quality_label,0,1,0,1);
    $Quality_table->attach_defaults ($Quality_spinner,1,2,0,1);
    #}}}3

    # Checkbox for Symlink Following {{{3
    my $Symlink_check = Gtk2::CheckButton->new ('_Follow Symlinks');
    $Symlink_check->signal_connect (clicked => sub {
            ($Symlink_check->get_active) and ($Symlink = 1);
            ($Symlink_check->get_active) or ($Symlink=0); });
    if ($Symlink) { $Symlink_check->set_active (TRUE); }
    else { $Symlink_check->set_active (FALSE); }
    #}}}3

    # Checkbox for Force {{{3
    my $Force_check = Gtk2::CheckButton->new ('Fo_rce');
    $Force_check->signal_connect (clicked => sub {
            ($Force_check->get_active) and ($Force = 1);
            ($Force_check->get_active) or ($Force = 0); });
    if ($Force) { $Force_check->set_active (TRUE); }
    else { $Force_check->set_active (FALSE); }
    # }}}3

    # Verbosity Radios {{{3
    my $quiet_verb_radio = Gtk2::RadioButton->new (undef,'quiet');
    my $detail_verb_radio = Gtk2::RadioButton->new_from_widget ($quiet_verb_radio,'normal');
    my $debug_verb_radio = Gtk2::RadioButton->new_from_widget ($quiet_verb_radio,'detailed');
    my $verb_radio_table = Gtk2::Table->new (1,4,FALSE);
    $verb_radio_table->set_col_spacings (5);
    my $verb_radio_label = Gtk2::Label->new_with_mnemonic ("_Print out Verbosity:");
    $verb_radio_label->set_mnemonic_widget ($quiet_verb_radio);

    # Connect Signals
    $quiet_verb_radio->signal_connect (clicked => sub {
            ($quiet_verb_radio->get_active) and ($Verbose = -1); });
    $detail_verb_radio->signal_connect (clicked => sub {
            ($detail_verb_radio->get_active) and ($Verbose = 0); });
    $debug_verb_radio->signal_connect (clicked => sub {
            ($debug_verb_radio->get_active) and ($Verbose = 1); });

    # Assign Pre-defined Values
    if ($Verbose == -1) { $quiet_verb_radio->set_active (TRUE); }
    elsif ($Verbose == 0) { $detail_verb_radio->set_active (TRUE); }
    elsif ($Verbose == 1) {$debug_verb_radio->set_active (TRUE); }

    $verb_radio_table->attach_defaults ($verb_radio_label,0,1,0,1);
    $verb_radio_table->attach_defaults ($quiet_verb_radio,1,2,0,1);
    $verb_radio_table->attach_defaults ($detail_verb_radio,2,3,0,1);
    $verb_radio_table->attach_defaults ($debug_verb_radio,3,4,0,1);
    #}}}3

    my $spin_hbox = Gtk2::HBox->new (TRUE,5);
    $spin_hbox->pack_start ($nice_table,FALSE,FALSE,0);
    $spin_hbox->pack_start ($Quality_table,FALSE,FALSE,0);

    my $sym_force_hbox = Gtk2::HBox->new (TRUE,5);
    $sym_force_hbox->pack_start ($Symlink_check,FALSE,FALSE,0);
    $sym_force_hbox->pack_start ($Force_check,FALSE,FALSE,0);
    
    $pref_left_vbox->pack_start ($fth_entry_table,FALSE,FALSE,0);
    $pref_left_vbox->pack_start ($Mode_radio_table,FALSE,FALSE,0);
    $pref_left_vbox->pack_start ($enc_radio_table,FALSE,FALSE,0);
    $pref_left_vbox->pack_start ($sym_force_hbox,FALSE,FALSE,0);

    $pref_right_vbox->pack_start ($oth_entry_table,FALSE,FALSE,0);
    $pref_right_vbox->pack_start ($spin_hbox,FALSE,FALSE,0);
    $pref_right_vbox->pack_start ($verb_radio_table,FALSE,FALSE,0);

    # Respone handling {{{3
    $pref_dialog->signal_connect (response => sub {
            if ($_[1] eq 'cancel') {
                $pref_dialog->destroy;
                return;
            }
            $Flac_Tree_Head = $fth_entry->get_text;
            $Output_Tree_Head = $oth_entry->get_text;
            $Nice_Value = $nice_spinner->get_value;
            $Quality = $Quality_spinner->get_value;
            if (verify_arguments()) {
                $pref_dialog->destroy;
                %Encode = ();
                @Purge = ();
                %Reencode = ();
                @Lim_Purge = ();
                Oggify::process();
                setup_list_display();
            }
        });
    #}}}3

    $pref_dialog->show_all;

}
#}}}2

# Run Dialog {{{2
sub run {
    # Prep the dialog box
    my $run_dialog = Gtk2::Dialog->new ('Running...', $window,
        'destroy-with-parent', 'gtk-cancel' => 'cancel');
    $run_dialog->set_default_response ('cancel');
    $run_dialog->set_default_size(450,100);
    $run_dialog->set_modal (TRUE);
    $run_dialog->set_resizable (TRUE);

    my $run_vbox = Gtk2::VBox->new (FALSE,10);

    my $progressbar = Gtk2::ProgressBar->new;
    $progressbar->set_orientation ('left-to-right');

    $run_dialog->vbox->pack_start ($run_vbox,FALSE,FALSE,0);
    $run_vbox->pack_start ($progressbar,FALSE,FALSE,0);

    my $run = 1;
    $run_dialog->signal_connect (response => sub {
            $run = 0;
            $progressbar->set_text ("Waiting for the last encode");
        });

    $run_dialog->show_all;

    my $total_operations = @{$guilist->{data}};
    my $increment = 1 / $total_operations;
    my $fraction = $progressbar->get_fraction;

    # The For loop that does the work {{{3
    for (my $x=0; $x < $total_operations; $x++) {
        $hold = 1;
        if (!$run) {
            $run_dialog->destroy;
            %Encode = ();
            @Purge = ();
            %Reencode = ();
            @Lim_Purge = ();
            Oggify::process ($Flac_Tree_Head, $Output_Tree_Head);
            setup_list_display();
            return;
        }

        # Ensure we don't overflow
        if ($fraction <= 1.0) {
            $progressbar->set_fraction($fraction);
            $fraction += $increment;
        }

        my $action = ${$guilist->{data}}[$x][1];
        my $src_file = ${$guilist->{data}}[$x][0];
        my $out_file = ${$guilist->{data}}[$x][2];

        my $file_name;
        $out_file =~ /.*\/(.+)\.(mp3|ogg)/;
        $file_name = "$1.$2";

        $progressbar->set_text($file_name);

        # Stupid gui updating loop
        while (Gtk2->events_pending) {
            Gtk2->main_iteration;
        }
        Gtk2::Gdk->flush;

        # pass action to the thread
        $queue->enqueue($action);

        if ($action eq "encode" or $action eq "re-encode") {
            $queue->enqueue($src_file);
            $queue->enqueue($out_file);
        } elsif ($action eq "re-tag") {
            $queue->enqueue($src_file);
            $queue->enqueue($out_file);
        } elsif ($action eq "purge") {
            $queue->enqueue($out_file);
        }

        # Loop to keep the gui responsive
        while ($hold) {
            while (Gtk2->events_pending) {
                Gtk2->main_iteration;
            }
            Gtk2::Gdk->flush;
            threads->yield();
        }

    }
    #}}}3
    $run_dialog->destroy;
    %Encode = ();
    @Purge = ();
    %Reencode = ();
    @Lim_Purge = ();
    Oggify::process();
    setup_list_display();
    return;
}
#}}}2

# Setup List Display {{{2
sub setup_list_display {
    @{$guilist->{data}} = ();
    if ($toolbar_encode->get_active) {
        add_encode();
    }
    if ($toolbar_reencode->get_active) {
        add_re('encode');
    }
    if ($toolbar_retag->get_active) {
        add_re('tag');
    }
    if ($toolbar_purge->get_active) {
        add_purge();
    }
    if ($toolbar_lpurge->get_active) {
        add_lim_purge();
    }
}
#}}}2

# Folder Selection Dialog {{{2
sub choose_folder {
    my $start = shift;

    my $file_dialog = Gtk2::FileChooserDialog->new ('Choose a Starting Folder',
        $window, 'select-folder',
        'gtk-cancel'    => 'cancel',
        'gtk-ok'    => 'ok');
    $file_dialog->set_select_multiple (FALSE);
    if (-e $start && -d $start) {
        $file_dialog->set_current_folder ($start);
    }

    my $response = $file_dialog->run;
    my $folder = $file_dialog->get_filename;
    $file_dialog->destroy;

    if ($response eq 'cancel') {
        return FALSE;
    } else {
        return $folder;
    }
}
#}}}2

# About Dialog {{{2
sub about_dialog {
    my $a_dialog = Gtk2::Dialog->new ('About Oggify', $window,
        'destroy-with-parent', 'gtk-close' => 'close');
    my $title = Gtk2::Label->new ();
    $title->set_use_markup (TRUE);
    $title->set_label ("<span size=\"larger\"><b>Gui-Oggify (goggify.pl)</b></span>");

    my $authors = Gtk2::Label->new ();
    $authors->set_label ("Author: Scott Paul Robertson <spr\@mahonri5.net>");

    my $version = Gtk2::Label->new ();
    $version->set_label ("Version: $version_num");

    my $a_vbox = Gtk2::VBox->new (FALSE,10);
    
    $a_vbox->pack_start ($title,FALSE,FALSE,0);
    $a_vbox->pack_start ($authors,FALSE,FALSE,0);
    $a_vbox->pack_start ($version,FALSE,FALSE,0);

    $a_dialog->vbox->pack_start ($a_vbox,FALSE,FALSE,0);

    $a_dialog->show_all;
    $a_dialog->run;
    $a_dialog->destroy;

}
#}}}2

# Help Dialog {{{2
sub help_dialog {
    my $help_dialog = Gtk2::Dialog->new ('Help', $window,
        'destroy-with-parent', 'gtk-close' => 'close');

    my $info = Gtk2::Label->new ();
    $info->set_line_wrap (TRUE);
    $info->set_use_markup (TRUE);
    $info->set_label ("<big>How to use Gui-Oggify (goggify.pl):</big>\n" .
        "Set a source location and out put location in preferences.\n" .
        "oggify will descend into both directories and identify possible files toencode, purge, or update. You can then see the actions that will take place when you press run. Encoding will take the flac file and encode it into mp3 or ogg, depending on your settings. Purge will delete the file or directory listed in the output column. Re-encode will do a fresh encode of an already existing file. Re-Tag will update the tags from the tags in the flac file. Clean will remove files that are going to be encoded, but already exist in a different format.");

    my $Quality = Gtk2::Label->new ();
    $Quality->set_use_markup (TRUE);
    $Quality->set_label ("How Quality Works with mp3:<tt>\n" .
    "----------------------------------------\n" .
    "| Mode |  quality  | lame setting      |\n" .
    "----------------------------------------\n" .
    "| cbr  |   0 - 1   | --preset cbr 64   |\n" .
    "| cbr  |     2     | --preset cbr 96   |\n" .
    "| cbr  |     3     | --preset cbr 128  |\n" .
    "| cbr  |     4     | --preset cbr 160  |\n" .
    "| cbr  |     5     | --preset cbr 192  |\n" .
    "| cbr  |     6     | --preset cbr 256  |\n" .
    "| cbr  |     7     | --preset insane   |\n" .
    "| vbr  |   0 - 2   | --preset medium   |\n" .
    "| vbr  |   3 - 5   | --preset standard |\n" .
    "| vbr  |   6 - 9   | --preset extreme  |\n" .
    "| abr  |   2 - 6   | same as cbr       |\n" .
    "| abr  |   7 - 9   | --preset abr 320  |\n" .
    "| any  |    10     | --preset insane   |\n" .
    "----------------------------------------\n" .
    "</tt>");

    my $nice = Gtk2::Label->new ();
    $nice->set_line_wrap (TRUE);
    $nice->set_use_markup (TRUE);
    $nice->set_label ("The Nice Value will nice the encoding actions with the given value, see the <tt>nice(1)</tt> manpage");

    my $oquality = Gtk2::Label->new ();
    $oquality->set_line_wrap (TRUE);
    $oquality->set_use_markup (TRUE);
    $oquality->set_label ("Quality in ogg mode follows the levels specified in the <tt>oggenc(1)</tt> manpage");

    my $sym = Gtk2::Label->new ();
    $sym->set_line_wrap (TRUE);
    $sym->set_use_markup (TRUE);
    $sym->set_label ("Following symlinks allows symlinks to be followed during scanning");

    my $force = Gtk2::Label->new ();
    $force->set_line_wrap (TRUE);
    $force->set_use_markup (TRUE);
    $force->set_label ("Force actions regardless of other checks. Purge non-empty directories.");

    my $etc = Gtk2::Label->new ();
    $etc->set_line_wrap (TRUE);
    $etc->set_use_markup (TRUE);
    $etc->set_label ("The 'scan' button updates the current listing. This automatically occurs after visiting the preferences dialog.\n" .
        "Known bugs: Canceling during a run will require the last action to finish before quitting. However, ctrl-c in a attached terminal will cancel encoding cleanly\n" .
        "Goggify is still in development, please contact the author with your feelings and suggestions. See the about dialog");

    my $help_vbox_left = Gtk2::VBox->new (FALSE,10);
    $help_vbox_left->pack_start ($info,FALSE,FALSE,0);
    $help_vbox_left->pack_start ($Quality,FALSE,FALSE,0);

    my $help_vbox_right = Gtk2::VBox->new (FALSE,10);
    $help_vbox_right->pack_start ($nice,FALSE,FALSE,0);
    $help_vbox_right->pack_start ($oquality,FALSE,FALSE,0);
    $help_vbox_right->pack_start ($sym,FALSE,FALSE,0);
    $help_vbox_right->pack_start ($force,FALSE,FALSE,0);
    $help_vbox_right->pack_start ($etc,FALSE,FALSE,0);

    my $help_hbox = Gtk2::HBox->new (FALSE,10);
    $help_hbox->pack_start ($help_vbox_left,TRUE,FALSE,0);
    $help_hbox->pack_start (Gtk2::VSeparator->new,FALSE,FALSE,0);
    $help_hbox->pack_start ($help_vbox_right,TRUE,FALSE,0);

    $help_dialog->vbox->pack_start ($help_hbox,FALSE,FALSE,0);
    $help_dialog->signal_connect (response => sub {
            $help_dialog->destroy;
        });
    $help_dialog->show_all;
}
#}}}2

sub warning_dialog {
    my $message = shift;

    my $dialog = Gtk2::MessageDialog->new ($window, 'destroy-with-parent',
        'error', 'ok', $message);
    $dialog->run;
    $dialog->destroy;
}

sub quit {
    save_rc_file();
    $queue->enqueue("end");
    $thr->join;
    Gtk2->main_quit;
    return TRUE;
}
#}}}1

# RC File Functions {{{1
sub load_rc_file {
    my $rcfile = $ENV{HOME} . "/.oggifyrc";

    if (-e $rcfile) {
        open(RCFILE, "< $rcfile") || die ("can't open rc file: $!");
    } else {
        return;
    }

    while ( <RCFILE> ) {
        if (/^fth = (.+)$/) {
            $Flac_Tree_Head = $1;
        } elsif (/^sth = (.+)$/) {
            $Output_Tree_Head = $1;
        } elsif (/^mode = (\.\w+)$/) {
            $Mode = $1;
        } elsif (/^not_mode = (\.\w+)$/) {
            $Not_Mode = $1;
        } elsif (/^enc_type = (\w+)$/) {
            $Enc_Type = $1;
        } elsif (/^quality = (\d+)$/) {
            $Quality = $1;
        } elsif (/^nice = (\d+)$/) {
            $Nice_Value = $1;
        } elsif (/^verb = (\d)$/) {
            $Verbose = $1;
        } elsif (/^sym = (\d)$/) {
            $Symlink = $1;
        }
    }

    close RCFILE;
}

sub save_rc_file {
    my $rcfile = $ENV{HOME} . "/.oggifyrc";

    open(RCFILE, "> $rcfile") || die ("can't open rc file: $!");

    print RCFILE "fth = $Flac_Tree_Head\n";
    print RCFILE "sth = $Output_Tree_Head\n";
    print RCFILE "mode = $Mode\n";
    print RCFILE "not_mode = $Not_Mode\n";
    print RCFILE "enc_type = $Enc_Type\n";
    print RCFILE "quality = $Quality\n";
    print RCFILE "nice = $Nice_Value\n";
    print RCFILE "verb = $Verbose\n";
    print RCFILE "sym = $Symlink\n";

    close RCFILE;
}

#}}}1

# Argument Verification {{{1
sub verify_arguments {
    $ready = FALSE;
    $toolbar_run->set_sensitive ($ready);
    if (!$Flac_Tree_Head) {
        warning_dialog ("You need to set the Flac Directory");
        return 0;
    }
    if (!(-e $Flac_Tree_Head && -d $Flac_Tree_Head)) {
        warning_dialog ("Source Directory needs to exist and be a directory");
        return 0;
    }
    if (!$Output_Tree_Head) {
        warning_dialog ("Need to set an Output Directory");
        return 0;
    }
    if ($Flac_Tree_Head eq $Output_Tree_Head) {
        warning_dialog ("Flac and Output Directories need to be different");
        return 0;
    }
    if ($Flac_Tree_Head =~ /(.*)\/$/) {
        if ($1 ne "") {
            $Flac_Tree_Head = $1;
        } else {
            $Flac_Tree_Head = "/";
        }
    }
    if ($Output_Tree_Head =~ /(.*)\/$/) {
        if ($1 ne "") {
            $Output_Tree_Head = $1;
        } else {
            $Output_Tree_Head = "/";
        }
    }
    my $temp = `which oggenc`;
    if ($Mode eq ".ogg" && !($temp)) {
        warning_dialog ("I need oggenc in the path for ogg encoding");
        exit(-1);
    }
    $temp = `which lame`;
    if ($Mode eq ".mp3" && !($temp)) {
        warning_dialog ("I need lame in the path for mp3 encoding");
        exit(-1);
    }
    $temp = `which flac`;
    if (!$temp) {
        warning_dialog ("I need flac in the path for encoding");
        exit(-1);
    }

    $ready = TRUE;
    $toolbar_run->set_sensitive ($ready);
    return 1;
}
#}}}1

# Thread for processing functions {{{1
sub command_runner {

    my $run = 1;
    # run until return
    while ($run) {
        # This call will block until the main thread sends the job
        my $job = $queue->dequeue;

        if ($job eq "encode" or $job eq "re-encode") {
            my $src = $queue->dequeue;
            my $out = $queue->dequeue;
            if ($Mode eq ".ogg") {
                Oggify::ogg_enc ($src, $out);
            } else {
                Oggify::mp3_enc ($src, $out);
            }
        } elsif ($job eq "re-tag") {
            my $src = $queue->dequeue;
            my $out = $queue->dequeue;
            Oggify::retag_file ($src, $out);
        } elsif ($job eq "purge") {
            my $out = $queue->dequeue;
            Oggify::purge ($out);
        } elsif ($job eq "end") {
            $run = 0
        }
        { lock $hold; $hold = 0; }
    }
    
}
#}}}1

# vim: sw=4 sts=4 expandtab
